##*****************************#
#******************************#
#***|                      |***#
#***|  BURIED IN OBLIVION  |***#
#***|      by Nisaba       |***#
#***|                      |***#
#******************************#
#*****************************##

%ext ck4


##****************************#
#***|  Temporary Patches  |***#
#****************************##

# Keen starts game in Level x
#%patch $5CBB $7A68W $0000W

# Don't end game if Keen reads
%patch $BC68 $EB   


#**************************#
#***|  Patching Stuff  |***#
#**************************#

# Map/Level files
%maphead MAPHEAD.CK4

# Tileinfo file
%patchfile $249C2 KEEN4.TLI

# Modified graphics
%egahead EGAHEAD.CK4

# Loading screen
%patchfile $1FE47 OBLIVION.BIN

# Startext file
%patchfile $1EE40 "OBLIVION.CK4"

# SaveState file
%patch $33434 "BURIED0x.CK4" $00


##************************************##
#***|  Swimming worldmap Keen fix  |***#
##************************************##

#touching INTILE 29 turns worldmap Keen into swimming worldmap Keen
%patch $E1EF
	$83 $FF 11		#	cmp	di, 11
	$7C $0F			#	jl	nope
	$83 $FF 14		#	cmp	di, 14
	$7E $0D			#	jle	shore
	$83 $FF 29		#	cmp	di, 29
	$75 $05			#	jne	nope
	$E9 $00BFw		#	jmp	setswim
	$90 $90			#	nop nop
				#nope:	jmp	return

#but only if Keen is not swimming (otherwise Keen might get stuck)
%patch $E2BC
	$EB $11			#	jmp	changestate
	$90 $90			#	nop nop
				#setswim:
	$81 $7C $1C $14CEw	#	cmp	[si+objtype.state], offset s_workdkeenswim
	$74 $14			#	je	return
	$C7 $44 $40 $0000w	#	mov	[si+objtype.temp2], 0


#******************************#
#***|  Screen Update Code  |***#
#******************************#

#-------------------------------------------------------------------------------
# Customized Screen Update Code For KEEN4 v1.4 (EGA)
# Copyright (c) 2019 K1n9_Duk3
#
# This software is provided 'as-is', without any express or implied warranty. In
# no event will the authors be held liable for any damages arising from the use
# of this software.
#
# Permission is granted to anyone to use this software for any purpose, including
# commercial applications, and to alter it and redistribute it freely, subject to
# the following restrictions:
#
# 1. The origin of this software must not be misrepresented; you must not claim
#    that you wrote the original software. If you use this software in a product,
#    an acknowledgment in the product documentation would be appreciated but is
#    not required.
#
# 2. Altered source versions must be plainly marked as such, and must not be
#    misrepresented as being the original software.
#
# 3. This notice may not be removed or altered from any source distribution.
#-------------------------------------------------------------------------------

#-------------------------------------------------------------------------------
# What does this patch script do? Well, it fixes a few minor glitches when
# playing COMMANDER KEEN 4-6 in DOSBox, such as:
#
# - jerky scrolling
# - sprites becoming invisible for one frame
# - sprites hidden behind foreground tiles becoming visible
#
# Changing the "Fix Jerky Motion" option in the options menu will change the
# behavior of the screen update code. This option should be turned OFF (which
# is the default setting) when playing in DOSBox.
#
# I also tested this code on various old PCs, using an MS-DOS 6.22 boot disk and
# running the game directly from DOS, and only one of them had problems with the
# default setting. That machine was a "Texas Instruments TravelMate 4000E 486
# WinSX/25MHz Color" system. Enabling the "Fix Jerky Motion" option fixed the
# problems on that machine.
#
# Note that this code has NOT been tested on an actual EGA card. The WaitVBL
# code was designed to stop waiting for a VBL when more than 1/70th of a second
# has passed since the code started to wait for a VBL. That's because the code
# can miss the VBL if the CPU was hadling an interrupt while the VBL signal was
# present and therefore the WaitVBL code didn't get to "see" the VBL signal.
# But EGA cards use a refresh rate of 60 Hz, not 70 Hz like the (S)VGA cards.
# That means it is possible for the WaitVBL code to exit a little too early on
# systems equipped with an EGA card, which means the issues listed above could
# still pop up.
#-------------------------------------------------------------------------------
# for use with K1n9_Duk3's Patching Utility:
#%exefile keen4e.exe 251712


#replace the v1.4 delay code with my custom VW_SetScreen code
#(my VW_SetScreen is bigger than the one in v1.4)
%patch $1E7F9
	$55			#	push	bp
	$8B $EC			#	mov	bp, sp
	$BA $03DAw		#	mov	dx, 3DAh
	$FA			#	cli
				#@l1:
	$EC			#	in	al, dx
	$A8 $01			#	test	al, 1
	$74 $FB			#	jz	@l1
				#@l2:
	$EC			#	in	al, dx
	$A8 $01			#	test	al, 1
	$75 $FB			#	jnz	@l2
	$8B $4E $06		#	mov	cx, [bp+arg_start]
	$BA $03D4w		#	mov	dx, 3D4h
	$B0 $0C			#	mov	al, 0Ch
	$EE			#	out	dx, al
	$42			#	inc	dx
	$8A $C5			#	mov	al, ch
	$EE			#	out	dx, al
	$4A			#	dec	dx
	$B0 $0D			#	mov	al, 0Dh
	$EE			#	out	dx, al
	$8A $C1			#	mov	al, cl
	$42			#	inc	dx
	$EE			#	out	dx, al
	$F7 $06 $7A14w $0001w	#	test	fixjerky, 1
	$74 $03			#	jz	@l3
	$E8 $0053w		#	call	WaitVBL
				#@l3:
	$F7 $06 $7ADEw $0001w	#	test	nopan, 1
	$75 $0C			#	jnz	l4
	$BA $03C0w		#	mov	dx, 3C0h
	$B0 $33			#	mov	al, 33h
	$EE			#	out	dx, al
	$EB $00			#	jmp	$
	$8A $46 $08		#	mov	al, [bp+arg_pan]
	$EE			#	out	dx, al
				#@l4:
	$F7 $06 $7A14w $0001w	#	test	fixjerky, 1
	$75 $03			#	jnz	@l5
	$E8 $0034w		#	call	WaitVBL
				#@l5:
	$FB			#	sti
	$5D			#	pop	bp
	$CB			#	retf

#replace the start of VW_SetScreen with a jump into my new code (see above):
%patch $1E87A
	$E9 $FF7Cw

#and overwrite the rest of the old VW_SetScreen code with my WaitVBL code:
	$8B $1E $A53Dw		#	mov	bx, word ptr TimeCount
	$BA $03DAw		#	mov	dx, 3DAh
				#@wait:	
	$FB			#	sti
	$EB $00			#	jmp	short $
	$FA			#	cli
	$EC			#	in	al, dx
	$A8 $08			#	test	al, 8
	$75 $0A			#	jnz	@done
	$A1 $A53Dw		#	mov	ax, word ptr TimeCount
	$2B $C3			#	sub	ax, bx
	$3D $0001w		#	cmp	ax, 1
	$76 $ED			#	jbe	@wait
				#@done:
	$C3			#	retn


#******************************#
#***|  Controlling Cheats  |***#
#******************************#

# Don't need debug mode to use F10 cheats
%patch $7AB7 $90 $90

# Levels F10 + W access
%patch $77E4 $11

# Warp Text
%patch $2F41A "Warp to which level(1-17):" $00

# Disable giving wetsuit on BAT & F10-I cheat
%patch $788F $EB $02 #BAT cheat
%patch $755C $EB $04 #F10-I cheat

# Stuff Keen has at game start
%patch $5CCD $7A58W $0000W #Ammo
%patch $5CC7 $7A6AW $0003W #Lives


#*********************#
#***|  Demo Mode  |***#
#*********************#

# Demo list length
%patch $3C0B $06

# Demo list location
%patch $3C16 $04E9W #$3CB9

# Main menu actions list
%patch $3CB9 $0448W #3C18: Terminator sequence & title screen
%patch $3CBB $046CW #3C3C: Star wars story
%patch $3CBD $045FW #3C2F: Demo 0 (Level 02 - NY00123)
%patch $3CBF $0473W #3C43: Demo 1 (Level 01 - ChargingMoose)
%patch $3CC1 $0481W #3C51: Demo 4 (Level 19 - High score)
%patch $3CC3 $0488W #3C58: Demo 2 (Level 05 - Szemigi)
%patch $3CC5 $0496W #3C66: Demo 3 (Level 10 - Candyjunk)

			  
#***********************#
#***|  Help Screen  |***#
#***********************#

# Help section documents
%patch $2F586 $1286W #About
              $1289W #Community Mods
              $1284W #The story
              $1285W #The game
              $1283W #Computer Wrist
			  
# Help menu option height
%patch $99DE $0018W #Height of hand options 1
%patch $99FC $0018W #Height of hand options 2
%patch $9A07 $0018W #Height of hand options 3	

%patch $99D7 $002DW #Hand (At 48,48)
%patch $99E3 $0030W #v location 1
%patch $99E7 $0025W #h location 1
%patch $9A0C $0030W #v location 2
%patch $9A10 $0025W #h location 2


#*************************#
#***|  Status Window  |***#
#*************************#

# Location text
%patch $2F487 "MOD" $00


#**************************************#
#***|  Didn't make it past window  |***#
#**************************************#

%patch $2F16F "You failed passing this" $00
%patch $2F191 "Exit to Map of Oblivion" $00


#*************************#
#***|  Bridge Switch  |***#
#*************************#

# Number of rows that make a bridge
%patch $C1F2 $0003W

#***********************#
#***|  Flying Foot  |***#
#***********************#

# Foot vertical spawning position
%patch $10FD6 $0C $0900W

# What picks the Foot destination to use
%patch $DE36 $7A1AW  $1400W $EB

# Foot location 1:
%patch $DE43 $06 #Horizontal
%patch $DE58 $0A #Vertical

# Location 2: (When exiting POTF)
%patch $DE78 $05 #Horizontal
%patch $DE8C $0A #Vertical


#*******************************#
#***|  water entering text  |***#
#*******************************#

# Text message for can't swim 
%patch $F3A5 $1BBCW # Pointer	
%patch $30A2C  "" $0A 
			   "Poisoned!" $00

# Got wetsuit message
%patch $F435 $1BC7W # Pointer	
%patch $30A37  "A safety suit." $0A
			   "Nice." $0A
			   "Let's dive!" $00


#***************************#
#***|  Princess Trixie  |***#
#***************************#

# Level where Keen gives the Miragia reply instead of the Pyramid one:
%patch $F033 $07 $75 # Desert Town 

# Lindsey's hint no 1:
%patch $EFA7 $1BA4W #Pointer	
%patch $30A14 "" $0A $00
%patch $3094C "Way south there's gear" $0A
              "to help passing the" $0A
              "polluted Quarry Pond." $00

# Keen's replies:
%patch $30848 $1B66W #Pointer	
%patch $309D6 "" $0A 
              "Polluted?" $0A 
              "Poisoned!" $00

# Lindsey's hint no 2:	
%patch $30846 $1B1EW #Pointer		  
%patch $3098E "To access the West" $0A
              "dig thru the" $0A
              "mountain range & follow" $0A 
              "the Footprints." $00

# Keen's replies:
%patch $3084A $1B7BW #Pointer	
%patch $309EB "" $0A
              "A Yeti Adventure?" $0A
              "Challenge accepted!" $00


#***********************************#
#***|  Oracle  Member Messages  |***#
#***********************************#

# Different Elder message in each level
%patch $F50D $8B $1E $A7EDW  $D1 $E3 $FF $B7 $3B59W  $9A $19310EBDRL     $83
             $C4 $02 $EB $0D

# Reply pointer list Oracle Member
%patch $329C9 $4080W  # Map
              $1D23W  # Level 1 Charging: "never visit" | "looking hale"
              $0000W  # Level 2
              $1D6DW  # Level 3 NY00123:   "Janitor?" | "Left Hand right Pocket"
              $4104W  # Level 4 Mark Candy:"Cucumber" | "Umbrella drink"
              $4159W  # Level 5 Szemigi:   "Promter"  | "Castle Moat"
              $1D6DW  # Level 6 Gargapplz: "Janitor?" | "Don't play it again"
              $41B1W  # Level 7
              $41B1W  # Level 8 Gridlock:  "Blueprints"     | "Canadian"
              $41E7W  # Level 9 
              $41E7W  # Level 10 Quirky:   "For God's sake" | "Toilet paper"
              $4212W  # Level 11 Shifted:  "Random number"  | "Cape?"
              $0000W  # Level 12
              $0000W  # Level 13
              $0000W  # Level 14
              $4080W  # Level 15

  
# Oracle Messages
 
#%patch $329C9 $1D23W # Level 1
%patch $30B93 "Hello, Commander!" $0A
              "It's been ages" $0A
              "since we've last met." $0A
              "You never visit..." $00
			  
#%patch $329C9 $1D6DW # Level 3, 6
%patch $30BDD "Keen! Here's still" $0A
              "lots of work to do!" $0A
              "If you come across" $0A
              "the Janitor, tell him" $0A
              "to head over!" $00

#%patch $329C9 $4104W # Level 4
%patch $32F74 "1 cucumber sliced" $0A
              "1 medium onion chopped" $0A
			  "1 teaspoon mustard seed" $0A
			  "a pinch of salt" $0A
			  "..." $00

#%patch $329C9 $4159W # Level 5
%patch $32FC9 "What a nice" $0A 
              "little castle moat," $0A
              "don't you think?" $0A
              "Once again the Janitor" $0A
              "outdid himself!" $00  
			  
#%patch $329C9 $41B1W # Level 8
%patch $33021  "" $0A 
               "... somehow" $0A 
			   "I must have lost" $0A 
			   "the construction plans." $00

#%patch $329C9 $41E7W # Level 10
%patch $33057 "" $0A
              "Oh for God's sake!" $0A
              "How did I get up here?" $00

#%patch $329C9 $4212W # Level 11
%patch $33082 "I'm thinking" $0A
			  "of a number" $0A
			  "between one and ten" $0A
			  "and I don't know why." $00


#**********************************#
#***|  Keen Replies To Elders  |***#
#**********************************#

# Keen's replies to elders depends on his current level
%patch $F580 $A7EDW # What is counted (Current level)
%patch $F586 $3EE5W # Location of text pointer list

# Keen Message list
%patch $32D55 $1C01W # Map
              $1BE9W # Level 1 Charging: "never visit" | "looking hale"
              $1C16W # Level 2
              $1C16W # Level 3 NY00123:   "Janitor?" | "Left Hand right Pocket"
              $1C5AW # Level 4 Mark Candy:"Cucumber" | "Umbrella drink"
              $1C8CW # Level 5 Szemigi:   "Promter"  | "Castle Moat"
              $1CAAW # Level 6 Gargapplz: "Janitor?" | "Don't play it again"
              $1CC8W # Level 7 
              $1CC8W # Level 8 Gridlock:  "Blueprints"     | "Canadian"
              $1D04W # Level 9 
              $1D04W # Level 10 Quirky:   "For God's sake" | "Toilet paper"
              $1D1AW # Level 11 Shifted:  "Random number"  | "Cape?"
              $1C16W # Level 12 
              $1C16W # Level 13 
              $1C16W # Level 14 
              $1C16W # Level 15

# Keen's replies: 

#%patch $32D55 $1BE9W # Level 1  
%patch $30A59 "" $0A 
              "You're looking hale" $0A
              "and hearty, lil' buddy!" $00
			  
#%patch $32D55 $1C1FW # Level 3 
%patch $30A86 "Have you ever tried" $0A
              "sticking your left hand" $0A
              "into your right pocket?" $00

#%patch $32D55 $1C63W # Level 4 
%patch $30ACA "I have this sudden" $0A
              "craving for an" $0A
              "umbrella drink." $00

#%patch $32D55 $1C95W # Level 5 
%patch $30AFC "Sure," $0A
			  "can I borrow" $0A
			  "your cape?" $00 
	
#%patch $32D55 $1D05W # Level 6
%patch $30B1A "" $0A
			  "...don't play it again," $0A
			  "Sam." $00

#%patch $32D55 $1CC9W # Level 8
%patch $30B38 "It's things like this" $0A
              "that make me wish" $0A
              "I were Canadian" $0A
              "..." $00

#%patch $32D55 $1CB3W # Level 10 
%patch $30B74 "" $0A
              "Out of toilet paper?" $00

#%patch $32D55 $1CC2W # Level 11
%patch $30B8A "" $0A
              "Me too!" $00


#*******************#
#***|  Janitor  |***#
#*******************#

# Level that Janitor appears in:
%patch $CD91 $12 $75

# Oracle Janitor wins level 
%patch $CD9E $C7 $06 $7A70W  $0002W  $90 $90 $90

# Janitor 1st window
%patch $F146 $1F53RW #Pointer
%patch $1F530 "You know," $0A 
              "being immortal" $0A
			  "is as boring as" $0A
              "a picturization" $0A
              "of an encyclopedia." $00
			  
# Janitor 2nd window 
%patch $F1B7 $1F5ERW #Text read from 1
%patch $1F5E0 "So we launched some" $0A
              "customized tours" $0A
              "and adventure trips." $0A
			  "Having fun?" $00
			  
# Keen 3rd window
%patch $F22A $1F4DRW # Pointer ($1F63RW)
%patch $1F4D0 "My innocence has been" $0A 
			  "shattered by this" $0A
			  "blatant tourist trap." $0A
			  "I want my money back." $00
			  
# Janitor 4th window
%patch $F29B $1F66RW # Pointer ($1F65RW)
%patch $1F660 "" $0A
              "You didn't pay anything." $00


#**********************#
#***|  High Score  |***#
#**********************#

# Position of names and scores
%patch $5A16 $0023W #Vertical location of table
%patch $5A1F $0018W #Horizontal position of names column
%patch $5A9A $0128W #Horizontal position of scores column

# Name entry position
%patch $5BE4 $0023W #Vertical
%patch $5BED $0018W #Horizontal

# Names:
%patch $33234 "Nisaba" $00
%patch $33274 "" $00
%patch $332B4 "GARGapplz" $00
%patch $332F4 "Gridlock" $00
%patch $33334 "Hagel" $00
%patch $33374 "Mark Candyjunk" $00
%patch $333B4 "NY00123" $00
%patch $333F4 "Charging Moose" $00

# Points: (10'000)
%patch $3326E $00002710L
%patch $332AE $00002710L
%patch $332EE $00002710L
%patch $3332E $00002710L
%patch $3336E $00002710L
%patch $333AE $00002710L
%patch $333EE $00002710L
%patch $3342E $00002710L

# Number of council members saved:
%patch $33272 $0001W
%patch $332B2 $0000W
%patch $332F2 $0002W
%patch $33332 $0003W
%patch $33372 $0004W
%patch $333B2 $0005W
%patch $333F2 $0006W
%patch $33432 $0007W


#***********************#
#***|  Music Files  |***#
#***********************#

# Music to use for Story Screen
%patch $5836 $000BW # (Too hot)

#Music to use for the Ending Sequence
%patch $9CD7 $0003W  # (Slug Village)

# Music played in each level:
#             VALUE   LEVEL                 MUSIC PLAYED         LEVEL No
%patch $2F206 $0000W  #Map                  (Shadowlands)          0
              $0004W  #Charging Moose / 1   (Kick in the Pants)    1
              $0003W  #NY00123 / Mirror     (Slug Village)         2
              $0003W  #NY00123 / Switches   (Slug Village)         3
              $0002W  #Mark Candy / Pit     (Too Hot)              4
              $0002W  #Szemigi / Allowed    (Too Hot)              5
              $0001W  #GARGapplz / Entry    (Eat Your Veggies)     6
              $0004W  #Szemigi / D-Town     (Kick in the Pants)    7
              $0002W  #Gridlock / Py-Maze   (Too Hot)              8
              $0002W  #Gridlock / Py-Poles  (Too Hot)              9
              $0004W  #Quirky Moron / Undrgr(Kick in the Pants)   10
              $0002W  #Shifted One / Py-Un  (Too Hot)             11
              $0001W  #Mark Candy / Oasis   (Eat your Veggies)    12
              $0004W  #Nisaba / Mires Peril (Kick in the Pants)   13
              $0002W  #Nisaba / Chiropody   (Too Hot)             14
              $0003W  #Well of Wisdom       (none)                15
              $0003W  #Gridlock / BwBerkloid(Slug Village)        16
              $0001W  #Nisaba / Diving      (Eat your Veggies)    17
              $0001W  #Mark & Nisaba / Fame (Eat your Veggies)    18


#***********************#
#***|  Level Names  |***#
#***********************#

# Level Names & Entry Texts

#Level 00
%patch $30764 $1F00037DRL #Name pointer Worldmap
%patch $1F37D "Worldmap Design by" $0A 
			  "Nisaba" $00
%patch $307C8 $1F000040RL #Pointer for Level text 00
%patch $1F040 "Keen returns to" $0A 
			  "the Mods of Oblivion" $00

#Level 01
%patch $30768 $1F000397RL #Pointer for Level name 01
%patch $1F397 "Level Design by" $0A 
			  "Charging Moose" $00			  
%patch $307CC $1F000065RL #Pointer for Level text 01
%patch $1F065 "Keen lurks at the" $0A 
			  "first ever modded" $0A 
			  "user level for K4" $00

#Level 02
%patch $3076C $1F0003B6RL #Pointer for Level name 02
%patch $1F3B6 "Level Design by" $0A 
			  "NY00123" $00
%patch $307D0 $1F00009BRL #Pointer for Level text 02
%patch $1F09B "Keen gets confused" $0A 
			  "by the Mirror Effect" $00

#Level 03
%patch $30770 $1F0003B6RL #Pointer for Level name 03
%patch $307D4 $1F0000C3RL #Pointer for Level text 03
%patch $1F0C3 "Keen is astonished" $0A 
			  "by the various usage" $0A 
			  "of Switches" $00

#Level 04
%patch $30774 $1F0003CERL #Pointer for Level name 04
%patch $1F3CE "Level Design by" $0A 
			  "Mark the Candyjunk" $00
%patch $307D8 $1F0000F7RL #Pointer for Level text 04
%patch $1F0F7 "Keen mines his way" $0A 
			  "through the" $0A 
			  "Pit of Passions" $00

#Level 05
%patch $30778 $1F0003F1RL #Pointer for Level name 05
%patch $1F3F1 "Level Design by" $0A 
			  "Szemigi" $00
%patch $307DC $1F000126RL #Pointer for Level text 05
%patch $1F126 "Keen hangs out in the" $0A 
			  "Castle of Allowed" $00

#Level 06
%patch $3077C $1F000409RL #Pointer for Level name 06
%patch $1F409 "Level Design by" $0A 
			  "GARGapplz" $00
%patch $307E0 $1F00014ERL #Pointer for Level text 06
%patch $1F14E "Keen uses the Entry" $0A 
			  "for some transition" $00

#Level 07
%patch $30780 $1F0003F1RL #Pointer for Level name 07
%patch $307E4 $1F000176RL #Pointer for Level text 07
%patch $1F176 "Keen finally discovers" $0A 
			  "the lost Desert Town" $00

#Level 08
%patch $30784 $1F000423RL #Pointer for Level name 08
%patch $1F423 "Level Design by" $0A 
			  "Gridlock" $00
%patch $307E8 $1F0001A2RL #Pointer for Level text 08
%patch $1F1A2 "Keen dares to enter" $0A 
			  "the Pyramid of Maze" $00

#Level 09
%patch $30788 $1F000423RL #Pointer for Level name 09
%patch $307EC $1F0001CARL #Pointer for Level text 09
%patch $1F1CA "Keen clambers the" $0A 
			  "Pyramid of Poles" $00
			  
#Level 10
%patch $3078C $1F00043CRL #Pointer for Level name 10
%patch $1F43C "Level Design by" $0A 
			  "Quirky Moron" $00
%patch $307F0 $1F0001EDRL #Pointer for Level text 10
%patch $1F1ED "Keen crawls into the" $0A 
			  "Undergroundpass" $00

#Level 11
%patch $30790 $1F000459RL #Pointer for Level name 11
%patch $1F459 "Level Design by" $0A 
			  "The Shifted One" $00
%patch $307F4 $1F000212RL #Pointer for Level text 11
%patch $1F212 "Keen moves silently in" $0A 
			  "the Unknown Pyramid" $00

#Level 12
%patch $30794 $1F0003CERL #Pointer for Level name 12
%patch $307F8 $1F00023DRL #Pointer for Level text 12
%patch $1F23D "Keen roams thru the" $0A 
			  "Palm Tree Oasis" $00

#Level 13
%patch $30798 $1F000479RL #Pointer for Level name 13
%patch $1F479 "Level Design by" $0A 
			  "Nisaba" $00
%patch $307FC $1F000261RL #Pointer for Level text 13
%patch $1F261 "Keen marches through" $0A 
			  "the Mires of Peril" $00

#Level 14
%patch $3079C $1F000479RL #Pointer for Level name 14
%patch $30800 $1F000289RL #Pointer for Level text 14
%patch $1F289 "Keen goes upwards" $0A 
			  "for some Chiropody" $00

#Level 15
%patch $307A0 $1F000490RL #Pointer for Level name 15
%patch $1F490 "Level Design by" $0A 
			  "Hagel & Nisaba" $00
%patch $30804 $1F0002AERL #Pointer for Level text 15
%patch $1F2AE "" $0A "Well, well..." $00

#Level 16
%patch $307A4 $1F000423RL #Pointer for Level name 16
%patch $30808 $1F0002BDRL #Pointer for Level text 16
%patch $1F2BD "Keen launches into the" $0A 
			  "Bean-with-Berkeloid" $0A 
			  "Rocket" $00

#Level 17
%patch $307A8 $1F000479RL #Pointer for Level name 17
%patch $3080C $1F0002EFRL #Pointer for Level text 17
%patch $1F2EF "Keen dives into" $0A 
			  "the strange world" $0A 
			  "of mutations" $00

#Level 18
%patch $307AC $1F0004AFRL #Pointer for Level name 18
%patch $1F4AF "Level Design by" $0A 
			  "Mark & Nisaba" $00
%patch $30810 $1F00031ERL #Pointer for Level text 18
%patch $1F31E "Keen gets lifted" $0A 
			  "up to the" $0A 
			  "Gallery of Fame" $00

#Level 19
%patch $307B0 $1F000371RL #High Scores
%patch $30814 $1F000349RL #Pointer for Level text 19
%patch $1F349 "Keen unexplainable" $0A 
			  "finds himself by" $0A 
			  "the High Scores" $00
			  

#*********************#
#***|  BWB / WOW  |***#
#*********************#

# BwB: Level 16 only 
%patch $60FD $10 $74

# Demo mode modified to switch from level 17 to 18 instead of map 
%patch $6B25 $83 $3E $A7EDW  $11 $74 $06 $C7 $06 $7A68W  $FFFFW  $FF $06 
          $7A68W $B8 $000DW  $50 $9A $187409F1RL     $83 $C4 $02   $A1 $A7EDW 
             $A3 $A7C2W  $8B $1E $A7EDW  $D1 $E3 $C7 $87 $7A1EW  $0001W  $EB $46

# W.O.W scrolling code -exit when off level top 
%patch $8553 $55 $8B $EC $83 $EC $02 $56 $57 $8B $76 $06 $83 $3E $A7DAW  $00 
             $74 $03 $E9 $012FW  $8B $44 $24 $3B $06 $C74BW  $77 $0C $C7 $06 
             $7A70W  $0002W  $5F $5E $8B $E5 $5D $CB 

                                 $A1 $C753W  $05 $0900W  $39 $44 $22 $73 $11 
             $8B $44 $22 $8B $16 $53 $C7 $81 $C2 $0900W  $2B $C2 $8B $F8 $EB 
             $21 $A1 $C753W  $05 $0C00W  $39 $44 $26 $76 $14 $8B $44 $26 $05 
             $10 $00 $8B $16 $53 $C7 $81 $C2 $0C00W  $2B $C2 $8B $F8 $EB $02 
             $33 $FF $A1 $C751W  $05 $00 $05 $39 $44 $24 $73 $12 $8B $44 $24 
             $8B $16 $51 $C7 $81 $C2 $00 $05 $2B $C2 $89 $46 $FE $EB $22 $A1 
             $C751W  $05 $00 $07 $39 $44 $28 $76 $12 $8B $44 $28 $8B $16 $51 
             $C7 $81 $C2 $00 $07 $2B $C2 $89 $46 $FE $EB $05 $C7 $46 $FE $00 
             $00 $0B $FF $75 $09 $83 $7E $FE $00 $75 $03 $E9 $0091W  $81 $FF 
             $00 $01 $7C $05 $BF $FF $00 $EB $09 $81 $FF $00 $FF $7F $03 $BF 
             $01 $FF $81 $7E $FE $00 $01 $7C $07 $C7 $46 $FE $FF $00 $EB $0C 
             $81 $7E $FE $00 $FF $7F $05 $C7 $46 $FE $01 $FF $FF $76 $FE $57 
             $9A $165413DDRL     $83 $C4 $04 $E9 $02B9W
			 
			
#**************************#
#***|  Keen vs Camera  |***#
#**************************#

# Disable EGAGRAPH check in Keen 4
%patch $3CE5 $90 $90

# New Keen spawn, 'I' Keen
%patch $EECB $00A9W #'I Keen' (At $E999)

# Change pointers to free up space
%patch $EE87 $0069W #Left Facing Keen (At $E959)
%patch $EE89 $0081W #Map Keen (At $E971)

# Compact Keen inits
%patch $E954 $B8 $0001W  $EB $03 $B8 $FFFFW  $50 $FF $76 $FC $57 $9A $0B80000ERL
                     $83 $C4 $06 $C7 $06 $CB3DW  $0001W  $EB $12
%patch $E971 $FF $76 $FC $57 $9A $0D8F050ARL     $83 $C4 $04 $C7 $06 $CB4FW
             $0001W  $9A $0D8F0003RL     $B8 $2A3ARW $8A $16 $EC $7A $8E $C0
             $26 $08 $16 $00FDW  $E9 $0464W
%patch $E999 $B8 $0001W  $50 $FF $76 $FC $57 $9A $037D0526RL     $EB $BE

# New Keen spawn=$037D0526RL
%patch $3CF6 $55 $8B $EC $8B $1E $D6 $A7 $C7 $07 $0002W  $C7 $47 $02 $0002W
             $C7 $47 $20 $0001W  $8B $46 $06 $B1 $08 $D3 $E0 $89 $47 $0A $8B
             $46 $08 $D3 $E0 $05 $FF0FW  $89 $47 $0C $8B $46 $0A $89 $47 $0E
             $C7 $47 $10 $0001W  $B8 $0CD4W  $50 $53 $9A $09DC118CRL     $83
             $C4 $04 $5D $CB

# New action sequence, Keen final position ticks, becomes Keen =$0CD4W
%patch $2FB44 $00D9W $00D9W $0000W $0000W $0000W $0006W $0000W $0000W #Make Keen
              $037D056ARL   $00000000L    $09DC176ERL   $013DW
%patch $2EFAD $00D9W $00D9W $0000W $0000W $0000W $006CW $0000W $0000W #Wait for Keen's arrival
              $00000000L    $00000000L    $09DC176ERL   $098CW

# Action behavior, create a Keen offscreen =$037D056ARL
%patch $3D3A $55 $8B $EC $56 $8B $76 $06 $B8 $0001W  $50 $9A $06BD1E11RL
             $83 $C4 $02 $8B $44 $0A $8B $1E $D8 $A7 $89 $47 $0A $8B $44 $0C
             $05 $04A0W  $89 $47 $0C $C7 $47 $10 $FFFFW  $C7 $07 $001FW  $C7 #Keen image depth =480
             $47 $20 $0001W 
             $8B $44 $0E $8B $1E $D8 $A7 $89 $47 $0E $B8 $011FW  $50 $53 $9A
             $09DC118CRL     $83 $C4 $04 $5E $5D $CB

# Keen action, rise up to meet Keen-final =$011FW
%patch $2EF8F $0082W $0082W $0001W $0000W $0000W $006DW $0000W $000BW
              $00000000L    $00000000L    $09DC176ERL   $0000W  


#***************#
#***|  WOW  |***#
#***************#

# If level 17 is done then replace all entrances to level 17 ($C011)
# with an entrance to level 18 ($C012).
# Note that PatchWorldMap is called after replacing the entrances, so
# once level 18 is done, that function will remove the level entrances.
%patch $3D88
				#seg001:05B8
	$83 $3E $A7EDw $00	#	cmp	mapon, 0
	$74 $01			#	je	scan
	$CB			#	retf
				#scan:
	$83 $3E $7A40w $00	#	cmp	gamestate.leveldone[17], 0
	$74 $22			#	je	done
	$56			#	push	si
	$1E			#	push	ds
	$A1 $A7E4w		#	mov	ax, mapwidth
	$F7 $2E	$A7E2w		#	imul	mapheight
	$89 $C1			#	mov	cx, ax
	$8E $1E $A7EBw		#	mov	ds, mapsegs+4
	$31 $F6			#	xor	si, si
	$FC			#	cld
				#planeloop:
	$AD			#	lodsw
	$3D $C011w		#	cmp	ax, 0C011h
	$75 $06			#	jne	nope
	$B8 $C012w		#	mov	ax, 0C012h
	$89 $44 $FE		#	mov	[si-2], ax
				#nope:
	$E2 $F2			#	loop	planeloop
	$1F			#	pop	ds
	$5E			#	pop	si
				#done:
	$9A $05C40484rl		#	call	PatchWorldMap
	$CB			#retf
	
#call the new code instead of checking the mapon variable and calling PatchWorldMap
%patch $6219
	$9A $037D05B8rl		#	call	myPatchWorldMap
	$90 $90 $90 $90 $90 $90
			  

#**************************#
#***|  Error Messages  |***#
#**************************#

# Free text space 32EF0 to 33189
#%patch $329C8 $00

# RF_NewMap: Map too tall!
%patch $16744  $4080W 
%patch $32EF0 "Map 2 tall" $00

# RF_MarkTileGraphics: Background anim of 0:
%patch $168F9  $408BW 
%patch $32EFB "Backgr anim 0" $00

# RF_MarkTileGraphics: Too many unique animated tiles!
%patch $16969  $4099W 
%patch $16B3A  $4099W 
%patch $32F09 "2 many anim tiles" $00

# RF_MarkTileGraphics: Unending background animation:
%patch $169F5  $40ABW 
%patch $32F1B "Unending backgr anim" $00

# RF_MarkTileGraphics: Foreground anim of 0:
%patch $16AC1  $40C0W 
%patch $32F30 "Foregr anim 0" $00

# RF_MarkTileGraphics: Unending foreground animation:
%patch $16BC6  $40CEW 
%patch $32F3E "Unending foregr anim" $00

# RF_CheckForAnimTile: No free spots in tilearray!
%patch $16CA1  $40E3W 
%patch $16D3A  $40E3W 
%patch $32F53 "AnimTile: 0 free tilearray spots" $00

# RFL_AnimateTiles: Out of bounds!
%patch $16F39  $4255W 
%patch $330C5 "AnimTiles: Out of bounds" $00

# RF_SetScrollBlock: Too many horizontal scroll blocks
%patch $1706B  $426EW 
%patch $330DE "2 many horizontal scroll blocks" $00

# RF_SetScrollBlock: Too many vertical scroll blocks
%patch $17094  $428EW 
%patch $330FE "2 many vertical scroll blocks" $00
			  
			  
%end